/*
 * @file Member ID Definition of ROUTING.
 *
 * Moderator: JasonJiang
 * Group ID: 0x00130000/0x80130000
 */

#ifndef _CSID_ROUTING_H
#define _CSID_ROUTING_H

#include "csid_gid.h"

#define _CFG_ROUTING(x)         (CSID_GIDC_ROUTING|(x))
#define _STA_ROUTING(x)         (CSID_GIDS_ROUTING|(x))

/* Extra Definitions */

#define ROUTING_DYNAMIC_DISABLE         0x00
#define ROUTING_DYNAMIC_RIP_V1          0x01
#define ROUTING_DYNAMIC_RIP_V2          0x02

#define MULTIROUTE_WAN_IDX				0xE0
#define MULTIROUTE_WAN1_IDX				0xE1


/* Configuration Items */
#define CSID_C_ROUTING_DYNAMIC_ENABLE		    _CFG_ROUTING(0x0001)	//T=u8,  enable/disable dynamic routing
#define CSID_C_ROUTING_STATIC_ENABLE		    _CFG_ROUTING(0x0002)	//T=u8,  enable/disable static routing 
#define CSID_C_MULTIROUTE_FAILCHG_ENABLE	    _CFG_ROUTING(0x0003)	//T=u32, D=1, enable/disable multi-wan fail change

/* For OSPF general setting */
#define CSID_C_ROUTING_OSPF_ENABLE			    _CFG_ROUTING(0x0010)    //T=u8,  enable/disable dynamic routing:OSPF
#define CSID_C_ROUTING_OSPF_BACKBONE		    _CFG_ROUTING(0x0011)    //T=str, OSPF backbone subnet
#define CSID_C_ROUTING_OSPF_DATA_STRING         _CFG_ROUTING(0x0012)    //T=str, OSPF data string, using save routing rule 

/* Customize for ZyXEL */
#define CSID_C_ROUTING_CUSTOMIZE_RULE		    _CFG_ROUTING(0x0020)	//T=u8, D=0, customize lan bind wan routing rule

/* For BGP general setting */
#define CSID_C_ROUTING_BGP_ENABLE			_CFG_ROUTING(0x0050)    //T=u8,  enable/disable dynamic routing:BGP
#define CSID_C_ROUTING_BGP_SELFID			_CFG_ROUTING(0x0051)    //T=u32,  BGP self ID value 
#define CSID_C_ROUTING_BGP_DATA_STRING		_CFG_ROUTING(0x0052)	//T=str,  BGP data string, using save routing rule

/* for Multi-WAN Multi-Routing Reserved 256 items*/
#define CSID_C_MULTIROUTE_ENABLE			_CFG_ROUTING(0x0100)	//T=u8, D=0, enable/disable multi routing rule
#define CSID_C_MULTIROUTE_PROTO				_CFG_ROUTING(0x0200)	//T=u8, D=0, specified protocol, 0:All, 1:TCP, 2:UDP, 3:Both
#define CSID_C_MULTIROUTE_WAN_INTERFACE		_CFG_ROUTING(0x0300)	//T=u32, D=0, Wan Interface	
#define CSID_C_MULTIROUTE_SRCIP_START		_CFG_ROUTING(0x0400)	//T=str, source start, IP/subnet
#define CSID_C_MULTIROUTE_SRCIP_END			_CFG_ROUTING(0x0500)	//T=str, source end, IP/subnet	
#define CSID_C_MULTIROUTE_DSTIP_START		_CFG_ROUTING(0x0600)	//T=str, destination start, IP/subnet/domain	
#define CSID_C_MULTIROUTE_DSTIP_END			_CFG_ROUTING(0x0700)	//T=str, destination end, IP/subnet/domain
#define CSID_C_MULTIROUTE_DSTPORT_START		_CFG_ROUTING(0x0800)	//T=u32, The start destination PORT
#define CSID_C_MULTIROUTE_DSTPORT_END		_CFG_ROUTING(0x0900)	//T=u32, The end destination PORT
/* end */

#define CSID_C_ROUTING_STATIC_RULE_ENABLE	_CFG_ROUTING(0x1000)	//T=u8, rule enable, 1:yes 0:no
#define CSID_C_ROUTING_STATIC_DEST_IP		_CFG_ROUTING(0x1100)	//T=ipv4, destination ip
#define CSID_C_ROUTING_STATIC_SUB_MASK		_CFG_ROUTING(0x1200)	//T=ipv4, subnet mask
#define CSID_C_ROUTING_STATIC_GW_IP		_CFG_ROUTING(0x1300)	//T=ipv4, gateway ip
#define CSID_C_ROUTING_STATIC_HOP		_CFG_ROUTING(0x1400)	//T=u16, numbers of hop
#define CSID_C_ROUTING_STATIC_IFNAME		_CFG_ROUTING(0x1500)	//T=str, interface name
#define CSID_C_ROUTING_STATIC_NAME		_CFG_ROUTING(0x1600)	//T=str, static routing name

/* For OSPF rule setting, reserved 256 items */
#define CSID_C_ROUTING_OSPF_RULE_ENABLE				_CFG_ROUTING(0x2100)    //T=u8,  enable/disable OSPF rules
#define CSID_C_ROUTING_OSPF_RULE_SUBNET				_CFG_ROUTING(0x2200)    //T=str, OSPF Area subnet
#define CSID_C_ROUTING_OSPF_RULE_ID					_CFG_ROUTING(0x2300)    //T=ipv4, OSPF Area ID


/* For BGP rule setting, reserved 256 items */
#define CSID_C_ROUTING_BGP_RULE_ENABLE		_CFG_ROUTING(0x3000)    //T=u8,  enable/disable BGP rules
#define CSID_C_ROUTING_BGP_RULE_IP			_CFG_ROUTING(0x3100)    //T=ipv4, BGP Neighbor IP
#define CSID_C_ROUTING_BGP_RULE_ID			_CFG_ROUTING(0x3200)    //T=u16, BGP Neighbor ID
//#define CSID_C_ROUTING_BGP_RULE_SUBNET		_CFG_ROUTING(0x7400)    //T=u16, BGP Out subnet

/* for Multi-WAN Multi-Routing extra configuration */
#define CSID_C_MULTIROUTE_SRCIP_TYPE        _CFG_ROUTING(0x4000)    //T=u8, D=0, 0:any, 1:subnet, 2:range, 3:single
#define CSID_C_MULTIROUTE_DSTIP_TYPE        _CFG_ROUTING(0x4100)    //T=u8, D=0, 0:any, 1:subnet, 2:range, 3:single, 4:domain
#define CSID_C_MULTIROUTE_DSTPORT_TYPE      _CFG_ROUTING(0x4200)    //T=u8, D=0, 0:all, 1:range, 2:single, 3:well-known app

/* Status Items */
#define CSID_S_ROUTING_STATIC_RULE_NUM		_STA_ROUTING(0x0001)	//T=u16, number of static routing rules
#define CSID_S_ROUTING_ALTERED				_STA_ROUTING(0x0002)	//T=u8, ui altered
#define CSID_S_ROUTING_RESTART_FLAG			_STA_ROUTING(0x0003)	//T=u16, D=0, check pptp pptpserver l2tp l2tp server if connect write 1, to run ripd restart write 0
#define CSID_S_ROUTING_WAN_NM_NUM			_STA_ROUTING(0x0004)	//T=u32, WAN subnet mask value

#endif //ifndef _CSID_ROUTING_H
