/*
 * @file Member ID Definition of Group SNMP.
 *
 * Moderator: JackChiang
 * Group ID: 0x00160000/0x80160000
 */

#ifndef _CSID_SNMP_H
#define _CSID_SNMP_H

#include "csid_gid.h"

#define _CFG_SNMP(x)		(CSID_GIDC_SNMP|(x))
#define _STA_SNMP(x)		(CSID_GIDS_SNMP|(x))

/* Configuration Items */
#define CSID_C_SNMP_LOCAL_ENABLE		_CFG_SNMP(0x0001) //T=u8, enable local snmp , 1:yes 0:no
#define CSID_C_SNMP_REMOTE_ENABLE       _CFG_SNMP(0x0002) //T=u8, enable remote snmp , 1:yes 0:no
#define CSID_C_SNMP_GET_COMMUNITY   	_CFG_SNMP(0x0003) //T=str, snmp community in get query
#define CSID_C_SNMP_SET_COMMUNITY   	_CFG_SNMP(0x0004) //T=str, snmp community in set action
#define CSID_C_SNMP_TRIP_IP	    		_CFG_SNMP(0x0005) //T=str, snmp send trip message to which ip 
#define CSID_C_SNMP_VERSION        		_CFG_SNMP(0x0009) //T=u8, snmp version of community
#define CSID_C_SNMP_SYSLOCATION			_CFG_SNMP(0x000a) //T=str, router's information in syslocation
#define CSID_C_SNMP_SYSCONTACT			_CFG_SNMP(0x000b) //T=str, router's information in syscontact
#define CSID_C_SNMP_SYSNAME				_CFG_SNMP(0x000c) //T=str, router's information in sysname
#define CSID_C_SNMP_WAN_ACCESS_IP		_CFG_SNMP(0x000d) //T=str, wan access IP address 
#define CSID_C_SNMP_SYSDESCR			_CFG_SNMP(0x000e) //T=str, router's information in descr
#define CSID_C_SNMP_ENABLE              _CFG_SNMP(0x000f) //T=u8, enable snmp , 1:yes 0:no

/* Configure SNMP Version */
#define CSID_C_SNMP_VERSION_V1          _CFG_SNMP(0x0010) //T=u8, snmp v1
#define CSID_C_SNMP_VERSION_V2C         _CFG_SNMP(0x0011) //T=u8, snmp v2c
#define CSID_C_SNMP_VERSION_V3          _CFG_SNMP(0x0012) //T=u8, snmp v3

/* Configure read/write SNMPv3 account */
#define CSID_C_SNMP_RW_ENABLE          	_CFG_SNMP(0x0013) //T=u8, enable local snmp , 1:yes 0:no
#define CSID_C_SNMP_RW_V3USER          	_CFG_SNMP(0x0014) //T=str, router's information in descr
#define CSID_C_SNMP_RW_V3PASSWORD      	_CFG_SNMP(0x0015) //T=str, router's information in descr
#define CSID_C_SNMP_RW_AUTHTYPE        	_CFG_SNMP(0x0016) //T=u8, D=0, MD5|SHA
#define CSID_C_SNMP_RW_PRIVACYMOD      	_CFG_SNMP(0x0017) //T=u8, D=1, noAuthNoPriv|authNoPriv|authPriv 
#define CSID_C_SNMP_RW_PRIVACY         	_CFG_SNMP(0x0018) //T=str, router's information in descr

/* Configure read only SNMPv3 account */
#define CSID_C_SNMP_RO_ENABLE           _CFG_SNMP(0x0019) //T=u8, enable local snmp , 1:yes 0:no
#define CSID_C_SNMP_RO_V3USER           _CFG_SNMP(0x001a) //T=str, router's information in descr
#define CSID_C_SNMP_RO_V3PASSWORD       _CFG_SNMP(0x001b) //T=str, router's information in descr
#define CSID_C_SNMP_RO_AUTHTYPE         _CFG_SNMP(0x001c) //T=u8, D=0, MD5|SHA
#define CSID_C_SNMP_RO_PRIVACYMOD       _CFG_SNMP(0x001d) //T=u8, D=1, noAuthNoPriv|authNoPriv|authPriv
#define CSID_C_SNMP_RO_PRIVACY          _CFG_SNMP(0x001e) //T=str, router's information in descr

/* Configure SNMPv3 multi account */
#define CSID_C_SNMP_MT_ENABLE           _CFG_SNMP(0x0030) //T=u8, enable multi account snmp, 1:yes,0:no
#define CSID_C_SNMP_MT_V3USER           _CFG_SNMP(0x0040) //T=str, snmpv3 user account
#define CSID_C_SNMP_MT_V3PASSWORD       _CFG_SNMP(0x0050) //T=str, snmpv3 user password
#define CSID_C_SNMP_MT_AUTHTYPE         _CFG_SNMP(0x0060) //T=u8, D=0, 0:MD5,1:SHA
#define CSID_C_SNMP_MT_PRIVACYMOD       _CFG_SNMP(0x0070) //T=u8, D=1, 0:noAuthNoPriv,1:authNoPriv,2:authPriv 
#define CSID_C_SNMP_MT_PRIVACY          _CFG_SNMP(0x0080) //T=str, DES passphrase
#define CSID_C_SNMP_MT_RO_RW            _CFG_SNMP(0x0090) //T=u8, 0:read only,1:read write

/* DDP DUT Info */
#define CSID_C_SNMP_DDPENABLE           _CFG_SNMP(0x001f) //T=u32,  enable ddp
#define CSID_C_SNMP_CPULOAD             _CFG_SNMP(0x0020) //T=u32,  CPU load
#define CSID_C_SNMP_MEMTOTAL			_CFG_SNMP(0x0021) //T=u32,  MEM total
#define CSID_C_SNMP_MEMFREE             _CFG_SNMP(0x0022) //T=u32,  MEM free
#define CSID_C_SNMP_MEMUSED             _CFG_SNMP(0x0023) //T=u32,  MEM used
#define CSID_C_SNMP_MEMLOAD             _CFG_SNMP(0x0024) //T=u32,  MEM used
#define CSID_C_SNMP_DDPBROADFREQ        _CFG_SNMP(0x0025) //T=u32,  ddp broadcast`s time period frequency
#define CSID_C_SNMP_DDPMAXNGTIME        _CFG_SNMP(0x0026) //T=u32,  max times of ddp reply 
#define CSID_C_SNMP_DDPUSER             _CFG_SNMP(0x0027) //T=str,  ddp client user
#define CSID_C_SNMP_DDPPSWD             _CFG_SNMP(0x0028) //T=str,  ddp client pswd
#define CSID_C_SNMP_NEWIP		_CFG_SNMP(0x0029) //T=ipv4, ddp client new ip
#define CSID_C_SNMP_DDPADD              _CFG_SNMP(0x002a) //T=u32,  1:fail 2:successful
#define CSID_C_SNMP_DDPSET              _CFG_SNMP(0x002b) //T=u32,  1:fail 2:successful
#define CSID_C_SNMP_DEVNUM              _CFG_SNMP(0x002c) //T=u32,  ddp dev num
#define CSID_C_SNMP_TRAPALTERED         _CFG_SNMP(0x002d) //T=u32,  1:send 
#define CSID_C_SNMP_POSTPATH            _CFG_SNMP(0x002e) //T=str,  ddp post path
#define CSID_C_SNMP_POSTIP              _CFG_SNMP(0x002f) //T=str,  ddp post ip

/* Configure SNMPv3 multi account */
#define CSID_C_SNMP_MT_ENABLE           _CFG_SNMP(0x0030) //T=u8, enable multi account snmp, 1:yes,0:no
#define CSID_C_SNMP_MT_V3USER           _CFG_SNMP(0x0040) //T=str, snmpv3 user account
#define CSID_C_SNMP_MT_V3PASSWORD       _CFG_SNMP(0x0050) //T=str, snmpv3 user password
#define CSID_C_SNMP_MT_AUTHTYPE         _CFG_SNMP(0x0060) //T=u8, D=0, 0:MD5,1:SHA
#define CSID_C_SNMP_MT_PRIVACYMOD       _CFG_SNMP(0x0070) //T=u8, D=1, 0:noAuthNoPriv,1:authNoPriv,2:authPriv
#define CSID_C_SNMP_MT_PRIVACY          _CFG_SNMP(0x0080) //T=str, DES passphrase
#define CSID_C_SNMP_MT_RO_RW            _CFG_SNMP(0x0090) //T=u8, 0:read only,1:read write

/* Configure SNMP Extra Items */
#define CSID_C_SNMP_PORT               		_CFG_SNMP(0x00a0) //T=u8, D=161, Set SNMP Port
#define CSID_C_SNMP_TRAP_ENABLE				_CFG_SNMP(0x00a1) //T=u8, 0:disable trap, 1:enable trap
#define CSID_C_SNMP_TRAP_COMMUNITY			_CFG_SNMP(0x00a2) //T=str, Set SNMP Trap Community
#define CSID_C_SNMP_WAN						_CFG_SNMP(0x00a3) //T=u8, SNMP WAN Index
#define CSID_C_SNMP_FWPATH					_CFG_SNMP(0x00a4) //T=u8, SNMP FIRMWARE PATH
#define CSID_C_SNMP_SECOND_TRAPPORT_ENABLE	_CFG_SNMP(0x00a5) //T=u8, SNMP swcondary trap port, 0:disable , 1: enable
#define CSID_C_SNMP_LAN_ACCESS_IP_ENABLE	_CFG_SNMP(0x00a6) //T=u8, for customer zyxel lan access ip access enable, disable

/*D-store Agent DDPv2*/
#define CSID_C_DSTORE_AGENT_SERVER_IP		_CFG_SNMP(0x2000) //T=ipv4, it DStore-anget sever IP
#define CSID_C_DSTORE_AGENT_SERVER_PORT		_CFG_SNMP(0x2002) //T=u32, it DStore-anget sever port

/* DDP Device Info */
#define CSID_S_SNMP_IPVERSION           _STA_SNMP(0x1000) //T=u32, ipVersion
#define CSID_S_SNMP_PRODUCTNAME         _STA_SNMP(0x1100) //T=str, productName
#define CSID_S_SNMP_FIRMEWAREVERSION    _STA_SNMP(0x1200) //T=str, firmewareVersion
#define CSID_S_SNMP_HARDWAREVERSION     _STA_SNMP(0x1300) //T=str, hardwareVersion 
#define CSID_S_SNMP_PRODUCTTYPE         _STA_SNMP(0x1400) //T=u32, productType
#define CSID_S_SNMP_PRODUCTHWSPEC       _STA_SNMP(0x1500) //T=u32, productHwSpecification
#define CSID_S_SNMP_MACADDRESS          _STA_SNMP(0x1600) //T=mac, macAddress
#define CSID_S_SNMP_SYSTEMNAME          _STA_SNMP(0x1700) //T=str, systemName
#define CSID_S_SNMP_SERIALNUMBER        _STA_SNMP(0x1800) //T=str, serialNumber
#define CSID_S_SNMP_PRODUCTCATEGORY     _STA_SNMP(0x1900) //T=str, productCategory
#define CSID_S_SNMP_DHCPSTATUS          _STA_SNMP(0x1a00) //T=u32, dhcpStatus
#define CSID_S_SNMP_ALERTREPORT         _STA_SNMP(0x1b00) //T=u32, 0/1 no-alert/alert happen
#define CSID_S_SNMP_IPADDRESS           _STA_SNMP(0x1c00) //T=ipv4, ipAddress
#define CSID_S_SNMP_DDPNONDDP           _STA_SNMP(0x1d00) //T=u32, ddp or none ddp
#define CSID_S_SNMP_WEBPORTNUMBER       _STA_SNMP(0x1e00) //T=u32, webPortNumber
#define CSID_S_SNMP_SPECIFYCUSTDDNS     _STA_SNMP(0x1f00) //T=u32, specifyCustomizedDns
#define CSID_S_SNMP_SUBNETMASK          _STA_SNMP(0x2000) //T=ipv4, subnetMask
#define CSID_S_SNMP_DEFAULTGATEWAY      _STA_SNMP(0x2100) //T=ipv4, defaultGateway
#define CSID_S_SNMP_PRIMARYDNS          _STA_SNMP(0x2200) //T=ipv4, primaryDns
#define CSID_S_SNMP_SECONDARYDNS        _STA_SNMP(0x2300) //T=ipv4, secondaryDns
#define CSID_S_SNMP_DEVICESTATUS        _STA_SNMP(0x2400) //T=u32, node status, inactive 1, active 2
#define CSID_S_SNMP_DEVICENGTIME        _STA_SNMP(0x2500) //T=u32, node no ack times
#define CSID_S_SNMP_DATETIME            _STA_SNMP(0x2600) //T=str, datetime
#define CSID_S_SNMP_TIMEZONE            _STA_SNMP(0x2700) //T=u32, timezone
#define CSID_S_SNMP_DST                 _STA_SNMP(0x2800) //T=u32, day-light saving, 0: disable | 1:enable
#define CSID_S_SNMP_PORTNUM             _STA_SNMP(0x2900) //T=str, port number: Port1/LAN1/WAN1/DMZ1
#define CSID_S_SNMP_SETTYPE             _STA_SNMP(0x2a00) //T=u32,  0:dhcp 1:static

/* Status Items */
#define CSID_S_SNMP_ALTERED             _STA_SNMP(0x0001) //T=u8, ui altered

/* DDP Status Items*/
#define CSID_S_SNMP_DDP_ALTERED         _STA_SNMP(0x0002) //T=u32,  ddp alter
#define CSID_S_SNMP_DDP_RELOAD			_STA_SNMP(0x0003) //T=u32, ddp reload altered
#define CSID_S_SNMP_DDP_SENDTRAP        _STA_SNMP(0x0004) //T=u32, 1:on -> off 2:off->on 
#define CSID_S_SNMP_DDP_TRAPMAC         _STA_SNMP(0x0005) //T=mac, sendtrap`s mac //00-50-18-12-34-56 -> 005018123456
#define CSID_S_SNMP_DDP_TRAPINDEX       _STA_SNMP(0x0006) //T=u32, sendtrap`s index

/*  CUSTOMER_ID       */
#define CSID_S_SNMP_CUSTOMER_ID			_STA_SNMP(0x0007) //T=u8, CUSTOMER_ID
/* SNMP  Multi flag */               
#define CSID_S_SNMP_MULTI				_STA_SNMP(0x0008) //T=u8, SNMP MULTI

/*SNMP another TRAP port*/
#define CSID_S_SNMP_TRAPPORT           	_STA_SNMP(0x0009) //T=u32, D=12823, Set another trap port for SNMP trap

#endif //ifndef _CSID_SNMP_H
