/*
 * udp.c - handle upd connections
 *
 * Copyright (C) 1999 Brad M. Garcia <garsh@home.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netdb.h>
#include <linux/if_ether.h>
#include <netinet/udp.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <assert.h>
#include "common.h"
#include "relay.h"
#include "cache.h"
#include "query.h"
#include "domnode.h"
#include "check.h"
#include "dns.h"

#ifndef EXCLUDE_MASTER
#include "master.h"
#endif
extern int private_dns_idx;
extern char private_dns[4][32];
extern char management_ip[32];
extern char private_domain[64];

int query_port;

static int udp_send_to_private(void *msg,int len,srvnode_t *srv)
{
    struct iphdr *ip; 
	struct udphdr *udp;
    struct sockaddr_in addr;
    struct hostent *host;
    int ip_head_len,udpsize;
    int rc,sockfd,on=1;
    char buffer[256];
    char *ptr;

    memset(&addr,0,sizeof(struct sockaddr_in));
    addr.sin_family=AF_INET; 
	addr.sin_port =(query_port);
    log_msg(LOG_DEBUG,"udp_socket_raw addr.sin_port=%d \n",addr.sin_port);
	host= (struct hostent *) gethostbyname((char *)(private_dns[private_dns_idx]));

	addr.sin_addr = *((struct in_addr *)host->h_addr);   
    
    if ((sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_UDP)) == -1) {
           perror("sockfd_tmp");
    }

    setsockopt(sockfd,IPPROTO_IP,IP_HDRINCL,&on,sizeof(on)); 
    ip_head_len=sizeof(struct iphdr); 
    
    bzero(buffer,256); 
    ip=(struct iphdr *)buffer; 
    ip->version=IPVERSION;
    ip->ihl=sizeof(struct ip)>>2;
    ip->tos=0;
    ip->tot_len=htons(ip_head_len);  
    ip->id=0;
    ip->frag_off=0;
    ip->ttl=128;
    ip->saddr=inet_addr(management_ip);
    ip->daddr =addr.sin_addr.s_addr;    ;
    log_msg(LOG_DEBUG,"ip->daddr =%x ip->saddr=%x\n",ip->daddr,ip->saddr);
    ip->protocol=IPPROTO_UDP;
    ptr=buffer+ip_head_len;
    
    udp=(struct udp *)ptr; 
    udpsize=sizeof(struct udphdr ) ;
    udp->source = (query_port);
    udp->dest = htons(53);
    udp->check = 0;
    udp->len = htons(udpsize+len);

    ptr+=udpsize;

    memcpy(ptr,msg,len);
#if 0
    int i=0;
    for(i=0;i<len+ip_head_len+udpsize;i++)
    {
        if(i!=0 && i%16==0)
               printf("\n");
        printf(" %02x ",buffer[i] & 0xff);        
    }
    printf("\n");
#endif
    rc = sendto(sockfd, buffer, len+ip_head_len+udpsize, 0,
		(struct sockaddr *)&addr,(socklen_t)sizeof(struct sockaddr_in));

     close(sockfd);

    return rc- udpsize - ip_head_len;
}

/*
 * dnssend()						22OCT99wzk
 *
 * Abstract: A small wrapper for send()/sendto().  If an error occurs a
 *           message is written to syslog.
 *
 * Returns:  The return code from sendto().
 */
static int udp_send(int sock, srvnode_t *srv, void *msg, int len)
{
    int	rc;
    char buf[256];
    //int isPrivateDomainName=0;
    //char *privateDomainName="flets.co.jp";
    
    time_t now = time(NULL); 
    memset(buf,0,256);
    snprintf_cname(msg, len, 12, buf, sizeof(buf));
    log_msg(LOG_DEBUG,"udp_send buf=%s\n",buf);
    log_msg(LOG_DEBUG,"udp_send port=%d\n",htons(srv->addr.sin_port));
    if(strstr(buf,private_domain)!=NULL && strlen(strstr(buf,private_domain)+strlen(private_domain))==0)
    {
       if(strlen(private_dns[0])==0 &&
          strlen(private_dns[1])==0 &&
          strlen(private_dns[2])==0 &&
          strlen(private_dns[3])==0 )
       {
            rc = sendto(sock, msg, len, 0,
		        (const struct sockaddr *) &srv->addr,
           		sizeof(struct sockaddr_in));
       }
       else
       {
            //rc=udp_send_to_private(msg,len,srv);
            //log_msg(LOG_DEBUG,"udp_send_to_private rc=%d\n",rc);
           if(inet_addr(private_dns[private_dns_idx]) != -1)
           {
            struct sockaddr_in addr;
           addr= srv->addr;
           addr.sin_addr.s_addr=inet_addr(private_dns[private_dns_idx]);
    
           rc = sendto(sock, msg, len, 0,
		         (const struct sockaddr *) &addr,
    		      sizeof(struct sockaddr_in));
        }  
            else
                return len;
        }  
    }
    else
    {
        rc = sendto(sock, msg, len, 0,
		    (const struct sockaddr *) &srv->addr,
    		sizeof(struct sockaddr_in));
    }    

    if (rc != len) {
	log_msg(LOG_ERR, "sendto error: %s: ",
		inet_ntoa(srv->addr.sin_addr), strerror(errno));
	return (rc);
    }
    if ((srv->send_time == 0)) srv->send_time = now;
    srv->send_count++;
    return (rc);
}

int send2current(query_t *q, void *msg, const int len) {
    /* If we have domains associated with our servers, send it to the
       appropriate server as determined by srvr */
  domnode_t *d;
  assert(q != NULL);
  assert(q->domain != NULL);
  log_msg(LOG_DEBUG, "udp bind port %d\n",q->port);
  query_port=q->port;
  d = q->domain;
  while ((d->current != NULL) && (udp_send(q->sock, d->current, msg, len) != len)) {
    if (reactivate_interval) deactivate_current(d);
  }
  if (d->current != NULL) {
    return len;
  } else return 0;
}


/*
 * handle_udprequest()
 *
 * This function handles udp DNS requests by either replying to them (if we
 * know the correct reply via master, caching, etc.), or forwarding them to
 * an appropriate DNS server.
 */
query_t *udp_handle_request()
{
    unsigned           addr_len;
    int                len;
    const int          maxsize = UDP_MAXSIZE;
    static char        msg[UDP_MAXSIZE+4];
    struct sockaddr_in from_addr;
    int                fwd;
    domnode_t          *dptr;
    query_t            *q, *prev;

    /* Read in the message */
    addr_len = sizeof(struct sockaddr_in);
    len = recvfrom(isock, msg, maxsize, 0, (struct sockaddr *)&from_addr, &addr_len);
    if (len < 0) {
        log_debug(1, "recvfrom error %s", strerror(errno));
        return NULL;
    }
    private_dns_idx=0;

    /* do some basic checking */
    if (check_query(msg, len) < 0) 
        return NULL;
 
    unsigned short trans_id = 0 ;
    int ret = 0; 
    fwd = -1;
    for (;;) {
        ret = handle_query(&from_addr, msg, &len, &dptr);
        if(ret < 0) {
            goto next_query;
        }
        else if(ret == 0) {
            // Send by cache 
            fwd = ret;
            break;
        }
        fwd = ret;

        /* Fetch the original Trans. ID of LAN PC */
        if(trans_id == 0){
            trans_id = *((unsigned short *)msg);
            //log_debug(1, "trans_id %x ", trans_id);
        }

        /* 
         *  dptr->current should never be NULL 
         *  it is checked in handle_query 
        */        
        /* rewrite msg, get id and add to list */
        if ((prev = query_add(dptr, dptr->current, &from_addr, msg, len)) == NULL) {
            /* of some reason we could not get any new queries. we have to drop this packet */
            goto next_query;
        }

        /* Save Trans. ID of LAN PC in query */
        q = prev->next;  
        if(trans_id > 0){
            q->trans_id = trans_id;
        }

        if (send2current(q, msg, len) > 0) {
            /* add to query list etc etc */
            goto next_query;
        } 
        else {
            /* we couldn't send the query */
            #ifndef EXCLUDE_MASTER
              int	packetlen;
              char	packet[maxsize+4];

              /*
               * If we couldn't send the packet to our DNS servers,
               * perhaps the `network is unreachable', we tell the
               * client that we are unable to process his request
               * now.  This will show a `No address (etc.) records
               * available for host' in nslookup.  With this the
               * client won't wait hang around till he gets his
               * timeout.
               * For this feature dnrd has to run on the gateway
               * machine.
               */
              if ((packetlen = master_dontknow(msg, len, packet)) > 0) {
                query_delete_next(prev);
                goto next_query;
                if (sendto(isock, msg, len, 0, (const struct sockaddr *)&from_addr, addr_len) != len) {
                    log_debug(1, "sendto error %s", strerror(errno));
                    goto next_query;
                }
              }
            #endif
        }  

        srvnode_t * s;
        next_query:
            if(dptr == NULL || dptr->current == NULL) {
                log_debug(1, "dptr == NULL or dptr->current == NULL !");               
                break;            
            }

            s = dptr->current->next;
            if(s != NULL) {
                if(s->inactive) {
                    dptr->current = NULL;
                    log_debug(1, "No any active DNS server, may the last server !");
                    break;            
                }                

                // Try next server
                log_debug(1, "Try the next DNS Server !");
                private_dns_idx++;
                dptr->current = s;                
            }
            else {
                log_debug(1, "The last DNS server !");
                break;
            }
    }

    if(fwd <= 0) {
        /* If we already know the answer, send it and we're done */
        if (sendto(isock, msg, len, 0, (const struct sockaddr *)&from_addr, addr_len) != len) {
            log_debug(1, "sendto error %s", strerror(errno));
        }
        return NULL; /* if its bogus, just ignore it */   
    }

    return q;
}

/*
 * dnsrecv()							22OCT99wzk
 *
 * Abstract: A small wrapper for recv()/recvfrom() with output of an
 *           error message if needed.
 *
 * Returns:  A positove number indicating of the bytes received, -1 on a
 *           recvfrom error and 0 if the received message is too large.
 */
static int reply_recv(query_t *q, void *msg, int len)
{
    int	rc, fromlen;
    struct sockaddr_in from;
    int i=0;//,isPrivateDomainName=0;
    //char *privateDomainName="flets.co.jp";
    char buf[256];

    fromlen = sizeof(struct sockaddr_in);
    rc = recvfrom(q->sock, msg, len, 0,
		  (struct sockaddr *) &from, &fromlen);

    if (rc == -1) {
	log_msg(LOG_ERR, "recvfrom error: %s",
		inet_ntoa(q->srv->addr.sin_addr));
	return (-1);
    }
    else if (rc > len) {
	log_msg(LOG_NOTICE, "packet too large: %s",
		inet_ntoa(q->srv->addr.sin_addr));
	return (0);
    }
    else if (memcmp(&from.sin_addr, &q->srv->addr.sin_addr,
		    sizeof(from.sin_addr)) != 0) {
		memset(buf,0,256);
		snprintf_cname(msg, len, 12, buf, sizeof(buf));    
        if(strstr(buf,private_domain)!=NULL && strlen(strstr(buf,private_domain)+strlen(private_domain))==0)
        {
           if(strlen(private_dns[0])==0 &&
              strlen(private_dns[1])==0 &&
              strlen(private_dns[2])==0 &&
              strlen(private_dns[3])==0 )
           {
                log_msg(LOG_WARNING, "unexpected server: %s",
    		        inet_ntoa(from.sin_addr));
    	        return (0);
           }
           else
           {
              for(i=0;i<4;i++)
              {
                log_msg(LOG_DEBUG,"from.sin_addr.s_addr =%08x inet_addr(private_dns[i])=%08x\n",from.sin_addr.s_addr,inet_addr(private_dns[i]));
                if (from.sin_addr.s_addr == inet_addr(private_dns[i])) 
		            return rc;        
              }
              log_msg(LOG_WARNING, "2unexpected server: %s",
    		        inet_ntoa(from.sin_addr));
    	        return (0);
           }      
    	
        }
        else
        {
            log_msg(LOG_WARNING, "unexpected server: %s",
    		        inet_ntoa(from.sin_addr));
    	        return (0);
        }
    }
    return (rc);
}

/*
 * handle_udpreply()
 *
 * This function handles udp DNS requests by either replying to them (if we
 * know the correct reply via master, caching, etc.), or forwarding them to
 * an appropriate DNS server.
 *
 * Note that the mached query is prev->next and not prev.
 */
void udp_handle_reply(query_t *prev)
{
    //const int          maxsize = 512; /* According to RFC 1035 */
    static char msg[UDP_MAXSIZE+4];
    int len;
    unsigned addr_len;
    query_t *q = prev->next;

    log_debug(3, "handling socket");
    if ((len = reply_recv(q, msg, UDP_MAXSIZE)) < 0) {
        log_debug(1, "dnsrecv failed: %i", len);
        goto end;
    }
    
    /* do basic checking */
    if (check_reply(q->srv, msg, len) < 0) {
      log_debug(1, "check_reply failed");
      goto end;
    }

    unsigned short num_ans = num_dns_ans(msg, len);
    unsigned short dns_flags = get_dns_flags(msg, len);
    if (opt_debug) {
        char buf[256];
        snprintf_cname(msg, len, 12, buf, sizeof(buf));
        log_debug(3, "Received DNS reply for \"%s\" (Ans: %d, Flag: 0x%x)", buf, num_ans, dns_flags);
    }
    
    dump_dnspacket("reply", msg, len);
    addr_len = sizeof(struct sockaddr_in);


    /* 
     * 1. A response with answers 
     * 2. If no any answers for all servers, including "No such name" response, 
     *    DNS relay must forward the last "No such name" response for the LAN PC.
     *    Otherwise, the LAN PC may lead to the endless waiting
     */
    if(q->trans_id == 0) {
        log_debug(2, "Already reply the answers, drop it !");    
        if(num_ans > 0) {
            /* Lets cache the reply and send to client */
            cache_dnspacket(msg, len, q->srv);
        } 
    }
    else if ((q->domain != NULL && num_ans > 0)           || 
             ((num_ans <= 0) && ((dns_flags & 0x3) == 0)) || // NO Error !
             (q->domain != NULL && num_ans == 0 && is_last_cliqid(q) > 0))  // Last DNS response
    {
        if(num_ans > 0) {
            /* Lets cache the reply and send to client */
            cache_dnspacket(msg, len, q->srv);
        }

        /* set the client qid */
        *((unsigned short *)msg) = q->client_qid;
        if(q->trans_id > 0) {
            *((unsigned short *)msg) = q->trans_id;
            log_debug(3, "Forwarding the reply to the host %s (Trans. ID: %04x)", 
                        inet_ntoa(q->client.sin_addr), q->trans_id);
        }
        else {
            log_debug(3, "Forwarding the reply to the host %s ", inet_ntoa(q->client.sin_addr));
        }

        mark_others_as_replied(q);
        if (sendto(isock, msg, len, 0, (const struct sockaddr *)&q->client, addr_len) != len) {
            log_debug(1, "sendto error %s", strerror(errno));
        }
    } 
    else if(num_ans <= 0) {
        log_debug(2, "A Reply without answers, drop it !");
    }
    else {
        log_debug(2, "We got a reactivation dummy reply. Cool!");
    }
      
    /* this server is obviously alive, we reset the counters */
    q->srv->send_time = 0;
    if (q->srv->inactive) {
        log_debug(1, "Reactivating server %s", inet_ntoa(q->srv->addr.sin_addr));
    }
    q->srv->inactive = 0;

end:    
    /* remove query from list and destroy it */
    query_delete_next(prev);
    return -1;
}


/* send a dummy packet to a deactivated server to check if its back*/
int udp_send_dummy(srvnode_t *s) {
  static unsigned char dnsbuf[] = {
  /* HEADER */
    /* we send a lookup for localhost */
    /* will this work on a big endian system? */
    0x00, 0x00, /* ID */
    0x01, 0x00, /* QR|OC|AA|TC|RD -  RA|Z|RCODE  */
    0x00, 0x01, /* QDCOUNT */
    0x00, 0x00, /* ANCOUNT */
    0x00, 0x00, /* NSCOUNT */
    0x00, 0x00, /* ARCOUNT */

    9, 'l','o','c','a','l','h','o','s','t',0,  /* QNAME */
    0x00,0x01,   /* QTYPE A record */
    0x00,0x01   /* QCLASS: IN */

    /* in case you want to lookup root servers instead, use this: */
    /*    0x00,       */ /* QNAME:  empty */
    /*    0x00, 0x02, */ /* QTYPE:  a authorative name server */
    /*    0x00, 0x01  */ /* QCLASS: IN */
  };
  query_t *q;
  struct sockaddr_in srcaddr;

  /* should not happen */
  assert(s != NULL);

  if ((q=query_add(NULL, s, &srcaddr, dnsbuf, sizeof(dnsbuf))) != NULL) {
    int rc;
    q = q->next; /* query add returned the query 1 before in list */
    /* don't let those queries live too long */
    q->ttl = reactivate_interval;
    memset(&srcaddr, 0, sizeof(srcaddr));
    log_debug(2, "Sending dummy id=%i to %s", ((unsigned short *)dnsbuf)[0], 
	      inet_ntoa(s->addr.sin_addr));
    /*  return dnssend(s, &dnsbuf, sizeof(dnsbuf)); */
    rc=udp_send(q->sock, s, dnsbuf, sizeof(dnsbuf));
    ((unsigned short *)dnsbuf)[0]++;
    return rc;
  }
  return -1;
}
